// $Header: /users/cvsleni/cvs/GA/galib/ga/gaarrayf.h,v 1.2 1999/09/15 14:13:14 adam Exp $
/* ----------------------------------------------------------------------------
  arraytmpl.h
  mbwall 22apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This defines the array base class.
---------------------------------------------------------------------------- */
#ifndef _ga_arraytmpl_h_
#define _ga_arraytmpl_h_

/* ----------------------------------------------------------------------------
This can be used only on objects that have a public operator= defined (because
of the resize).
Must have a creation operator defined that takes no args
must have operator == and operator != defined

We don't do any over-allocation, so resizing can be expensive.
No error checking on the copy, so don't walk over end of array!

*** should do specialization for simple types that does memcpy rather than loop
---------------------------------------------------------------------------- */

#ifndef T
#define T float
#endif

class GAArray {
public:
  GAArray(unsigned int s) : sz(s), a(sz ? new T[sz] : 0){}
  GAArray(const GAArray & orig){sz=0; a=NULL; copy(orig);}
  GAArray & operator=(const GAArray & orig){copy(orig); return *this;}
  GAArray & operator=(const T array []) // no err checks!
    {for(unsigned int i=0; i<sz; i++) a[i] = array[i]; return *this;}
  virtual ~GAArray(){delete [] a;}

  GAArray * clone(){return new GAArray(*this);}
  operator const T * () const {return a;}
  operator T * () {return a;}
  const T & operator[](unsigned int i) const {return a[i];}
  T & operator[](unsigned int i) {return a[i];}
  void copy(const GAArray & orig){
    size(orig.sz);
    for(unsigned int i=0; i<sz; i++)
      a[i] = orig.a[i];
  }
  void copy(const GAArray & orig, unsigned int dest,
	    unsigned int src, unsigned int length){
    for(unsigned int i=0; i<length; i++) a[dest+i] = orig.a[src+i];
  }
  void move(unsigned int dest, unsigned int src, unsigned int length){
    if(src > dest)
      for(unsigned int i=0; i<length; i++) a[dest+i] = a[src+i];
    else if(src < dest)
      for(unsigned int i=length-1; i!=0; i--) a[dest+i] = a[src+i];
  }
  void swap(unsigned int i, unsigned int j){T tmp=a[j]; a[j]=a[i]; a[i]=tmp;}
  int size() const {return sz;}
  int size(unsigned int n){
    if(n == sz) return sz;
    T * tmp = (n ? new T[n] : 0);
    for(int i=((n < sz) ? n-1 : sz-1); i>=0; i--) tmp[i] = a[i];
    delete [] a;
    a = tmp;
    return sz=n;
  }
  int equal(const GAArray & b,
	    unsigned int dest, unsigned int src, unsigned int length) const {
    for(unsigned int i=0; i<length; i++)
      if(a[dest+i] != b.a[src+i]) return 0;
    return 1;
  }
friend  inline int operator==(const GAArray & a, const GAArray & b);
friend  inline int operator!=(const GAArray & a, const GAArray & b);

protected:
  unsigned int sz;		// number of elements
  T * a;			// the contents of the array
};

inline int operator==(const GAArray & a, const GAArray & b){
  if(a.size() != b.size()) return 0;
  return a.equal(b,0,0,a.size());
}
inline int operator!=(const GAArray & a, const GAArray & b){
  if(a.size() != b.size()) return 1;
  return a.equal(b,0,0,a.size()) ? 0 : 1;
}

#undef T

#endif


