#include "ga/MixedGenome.h"


// Blend crossover generates a new value based on the interval between parents.
// We generate a uniform distribution based on the distance between parent
// values, then choose the child value based upon that distribution.
// modified Adam 23Nov96 to always obey limits
// on alleles.

int GAListBlendCrossover(const GAGenome& p1, const GAGenome& p2,
		     GAGenome* c1, GAGenome* c2) {
  GA1DArrayAlleleGenome &mom=(GA1DArrayAlleleGenome &)p1;
  GA1DArrayAlleleGenome &dad=(GA1DArrayAlleleGenome &)p2;
  register int i,j;
  int n=0;

  if(c1 && c2){
    GA1DArrayAlleleGenome &sis=(GA1DArrayAlleleGenome &)*c1;
    GA1DArrayAlleleGenome &bro=(GA1DArrayAlleleGenome &)*c2;

    int len = mom.length();	// assumes both same length
    int dist,momindx,dadindx,found;
    for(i=0; i<len; i++) {	// for each gene
      found=0;
      dist=0;
      for(j=0;j<mom.alleleset(i).size()-1;j++) {
	if(mom.gene(i) == mom.alleleset(i).allele(j)) {
	  momindx = j;
	  found++;
	  if(found == 2)
	    break;
	}
	if(dad.gene(i) == dad.alleleset(i).allele(j)) {
	  dadindx = j;
	  found++;
	  if(found == 2)
	    break;
	}
      }
				// now have the two indx for the parents
      if(dadindx > momindx) 
	dist = dadindx-momindx;
      else
	dist = momindx-dadindx;
      
      if(dist > 1) {
	dist =  dist/2;
      }
      
      int lo = GAMax((int)0,(int)GAMin(dadindx,momindx)-dist);
      int hi = GAMin(mom.alleleset(i).size()-1,(int)GAMin(dadindx,momindx)+dist);
      sis.gene(i,mom.alleleset(i).allele(GARandomInt(lo,hi)));
      bro.gene(i,mom.alleleset(i).allele(GARandomInt(lo,hi)));
    }
    n = 2;
  }
  else if(c1 || c2){
    GA1DArrayAlleleGenome &sis=
      (c1 ? (GA1DArrayAlleleGenome &)*c1:(GA1DArrayAlleleGenome &)*c2);

    int len = mom.length();	// assumes both same length
    int dist,momindx,dadindx,found;
    for( i=0; i<len; i++) {	// for each gene
      found=0;
      dist=0;
      for(j=0;j<mom.alleleset(i).size()-1;j++) {
	if(mom.gene(i) == mom.alleleset(i).allele(j)) {
	  momindx = j;
	  found++;
	  if(found >1)
	    break;
	}
	if(dad.gene(i) == dad.alleleset(i).allele(j)) {
	  dadindx = j;
	  found++;
	  if(found >1)
	    break;
	}
      }
				// now have the two indx for the parents
      if(dadindx > momindx) 
	dist = dadindx-momindx;
      else
	dist = momindx-dadindx;
      if(dist > 1) {
	dist =  dist/2;
      }
      int lo = GAMax((int)0,(int)GAMin(dadindx,momindx)-dist);
      int hi = GAMin(mom.alleleset(i).size()-1,(int)GAMin(dadindx,momindx)+dist);
      sis.gene(i,mom.alleleset(i).allele(GARandomInt(lo,hi)));
    }
    n = 1;
  }

  return n;
}


// Blend crossover generates a new value based on the interval between parents.
// We generate a uniform distribution based on the distance between parent
// values, then choose the child value based upon that distribution.
// modified Adam 23Nov96 to always obey limits
// on alleles.
int 
GARealBlendCrossover(const GAGenome& p1, const GAGenome& p2,
		     GAGenome* c1, GAGenome* c2) {
  GA1DArrayAlleleGenome &mom=(GA1DArrayAlleleGenome &)p1;
  GA1DArrayAlleleGenome &dad=(GA1DArrayAlleleGenome &)p2;

  int n=0;

  if(c1 && c2){
    GA1DArrayAlleleGenome  &sis=(GA1DArrayAlleleGenome  &)*c1;
    GA1DArrayAlleleGenome  &bro=(GA1DArrayAlleleGenome  &)*c2;

    int len = GAMax(mom.length(), dad.length());
    for(int i=0; i<len; i++) {
      float dist = 0;
      if(mom.gene(i) > dad.gene(i)) 
		dist = mom.gene(i) - dad.gene(i);
      else 
		dist = dad.gene(i) - mom.gene(i);
      float lo = GAMax(((float)(GAMin(mom.gene(i), dad.gene(i))) - 0.5*dist),(float)mom.alleleset(i).lower());
      float hi = GAMin(((float)(GAMax(mom.gene(i), dad.gene(i))) + 0.5*dist),(float)mom.alleleset(i).upper());
      sis.gene(i, GARandomFloat(lo, hi));
      bro.gene(i, GARandomFloat(lo, hi));
    }
    n = 2;
  }
  else if(c1 || c2){
    GA1DArrayAlleleGenome &sis = 
      (c1 ? (GA1DArrayAlleleGenome &)*c1 : (GA1DArrayAlleleGenome &)*c2);

    int len = GAMax(mom.length(), dad.length());
    for(int i=0; i<len; i++) {
      float dist = 0;
      if(mom.gene(i) > dad.gene(i)) 
	dist = mom.gene(i) - dad.gene(i);
      else 
	dist = dad.gene(i) - mom.gene(i);
      float lo = GAMax(((float)(GAMin(mom.gene(i), dad.gene(i))) - 0.5*dist),(float)mom.alleleset(i).lower());
      float hi = GAMin(((float)(GAMax(mom.gene(i), dad.gene(i))) + 0.5*dist),(float)mom.alleleset(i).upper());
      sis.gene(i, GARandomFloat(lo, hi));
    }
    n = 1;
  }

  return n;
}


// Member functions for the composite genome object
MixedGenome::MixedGenome(GAAlleleSetArray &float_allele,GAAlleleSetArray &int_allele, GAGenome::Evaluator f, void* u):
GAGenome(MixedInitializer,
	 MixedMutator,
	 MixedComparator) 
{
    evaluator(f); 
    userData(u); 
    crossover(MixedCrossover);
    _greal = new GA1DArrayAlleleGenome (float_allele,  f, u);
    _greal->crossover(GARealBlendCrossover);
    _gint = new GA1DArrayAlleleGenome (int_allele,  f, u);
    _gint->crossover(GAListBlendCrossover);
}

MixedGenome::MixedGenome(const MixedGenome & orig) {
  _greal = new GA1DArrayAlleleGenome (orig.real_genome());
  _gint = new GA1DArrayAlleleGenome (orig.int_genome());
  copy(orig);
}
  
MixedGenome& MixedGenome::operator=(const GAGenome& g) { copy(g); return *this; }

MixedGenome::~MixedGenome() { delete _greal; delete _gint; }

GAGenome* MixedGenome::clone(GAGenome::CloneMethod) const {
  return new MixedGenome(*this);
}

void MixedGenome::copy(const GAGenome & c){
  if(&c != this && sameClass(c)){
    GAGenome::copy(c);
    MixedGenome & bc = (MixedGenome &)c;
    _greal->copy(*(bc._greal));
    _gint->copy(*(bc._gint));
  }
}

int MixedGenome::equal(const GAGenome& g) const {
  MixedGenome& genome = (MixedGenome&)g;
  GA1DArrayAlleleGenome &g1 = (GA1DArrayAlleleGenome &) (this->real_genome());
  GA1DArrayAlleleGenome &g2 = (GA1DArrayAlleleGenome &) (genome.real_genome());
  GA1DArrayAlleleGenome &h1 = (GA1DArrayAlleleGenome &) (this->int_genome());
  GA1DArrayAlleleGenome &h2 = (GA1DArrayAlleleGenome &) (genome.int_genome());
  return(g1.equal(g2) && h1.equal(h2));
//  return (((*_greal) == (*(genome._greal))) && ((*_gint) == (*(genome._gint))));
}

int MixedGenome::read(istream & is) {
  float sc=0.0;

  is>>*_greal>>*_gint;
  is>>sc;
  this->score(sc);// set the score
  return is.fail() ? 1 : 0; 
}

int MixedGenome::write(ostream & os) const {
  
  os<< *_greal << *_gint;
  os << _score;

  return os.fail() ? 1 : 0;
}



// These are the default initialization, mutation, and comparator operators for
// this genome class.  They are defined as static functions of the composite
// genome class and they're defaults for the class.  But they can be overridden
// on any instance of the genome.

// The initializer just calls the initializer for each of the genomes that are
// in the composite genome.

// I would have used simply 'Initializer', 'Mutator', etc rather than
// 'CompositeInitializer' but old versions of g++ are brain-dead and don't
// get the encapsulation properly.
void MixedGenome::MixedInitializer(GAGenome & c) {
  MixedGenome & child = (MixedGenome &)c;
  child.real_genome().initialize();
  cerr << "OK\n";
  child.int_genome().initialize();
}


// The mutator just calls the mutator for each of the component genomes.
int MixedGenome::MixedMutator(GAGenome & c, float pmut) {
  MixedGenome & child = (MixedGenome &)c;
  return child.real_genome().mutate(pmut) + child.int_genome().mutate(pmut);
}

// The comparator just calls the comparators for each of the component genomes,
// then averages the score.
float MixedGenome::MixedComparator(const GAGenome& a, const GAGenome& b) {
  fprintf(stderr,"called MixedComparator - NOT WRITTEN YET !!!!\n");
  exit(2);
  MixedGenome& sis = (MixedGenome &)a;
  MixedGenome& bro = (MixedGenome &)b;
  //  return 0.5 * (sis.binstr().compare(bro) + sis.bin2dec().compare(bro));
  return(0);
}

// The crossover operator invokes the crossover for each of the genomes in the
// composite genome.  We use sexual crossover only, and we do not test to see
// if no crossover has been assigned.
int
MixedGenome::
MixedCrossover(const GAGenome& a, const GAGenome& b,
		   GAGenome* c, GAGenome* d){ 
  MixedGenome& mom = (MixedGenome&)a;
  MixedGenome& dad = (MixedGenome&)b;
  int n=0;

  GAGenome::SexualCrossover realcross = mom._greal->sexual();
  GAGenome::SexualCrossover intcross = mom._gint->sexual();

  if(c && d){
    MixedGenome& sis = (MixedGenome&)*c;
    MixedGenome& bro = (MixedGenome&)*d;
    (*realcross)(mom.real_genome(), dad.real_genome(), &sis.real_genome(), &bro.real_genome());
    (*intcross)(mom.int_genome(),dad.int_genome(), &sis.int_genome(), &bro.int_genome());
    n = 2;
    sis._evaluated= gaFalse;
    bro._evaluated= gaFalse;
  }
  else if(c){
    MixedGenome& sis = (MixedGenome&)*c;
    (*realcross)(mom.real_genome(), dad.real_genome(), &sis.real_genome(), 0);
    (*intcross)(mom.int_genome(), dad.int_genome(), &sis.int_genome(), 0);
    n = 1;
    sis._evaluated= gaFalse;
  }
  else if(d){
    MixedGenome& bro = (MixedGenome&)*d;
    (*realcross)(mom.real_genome(), dad.real_genome(), &bro.real_genome(), 0);
    (*intcross)(mom.int_genome(), dad.int_genome(), &bro.int_genome(), 0);
    n = 1;
    bro._evaluated= gaFalse;
  }
  return n;
}










