//////////////////////////////////////////////////////////////////////
// MLInterface.cpp: implementation of the MLInterface class.
// base class for mathlib interface
//
// see MLInterface for explanation header
//////////////////////////////////////////////////////////////////////

#include "mex.h"
#include "MLInterface.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MLInterface::MLInterface()
{
	m_Clone = false;
}


MLInterface::~MLInterface()
{
	
}

// Associate a mxarray input argument to the interface we create a clone 
// of this interface to be able to change the values and return new values
// to the callee.


void MLInterface::SetArgs(const mxArray *Arg )
{
	SetArgs(Arg, false);
}

void MLInterface::SetArgs(const mxArray *Arg, bool Clone )
{
	printf("setArg\n");
	m_ArgNorm = Arg;
	m_Clone = Clone;
	if( !Clone )
		return;
	
	printf("setArg on clone\n");

	int i, numfields,num_dims;
	const int *dims;
	const char **fnames;

	// get all the structures specs
	num_dims = mxGetNumberOfDimensions(Arg);
	dims = mxGetDimensions(Arg);
	numfields = mxGetNumberOfFields(Arg);
	fnames = (const char **)mxCalloc(numfields,sizeof(char **));
	for( i=0; i < numfields; i++)
		fnames[i] = mxGetFieldNameByNumber(Arg,i);
	
	// creating the clone structure
	m_ArgClone = mxCreateStructArray(num_dims,dims,numfields,fnames);

	// copying all the fields
	for(i=0; i < numfields;i++)
		{
		mxArray *tmp;
		char * buff;
		int status,bufflen;

		// reading each file
		tmp = mxGetField(Arg,0,fnames[i]);

		printf("setArg on copie %s\n", fnames[i]);


		// test the type
		if(!mxIsDouble(tmp))
			{
			// copying strings
			bufflen = mxGetN(Arg)*mxGetM(Arg)+1;
			buff = (char *)mxCalloc(bufflen, sizeof(char)); // +1 for the \0
			status = mxGetString(tmp,buff,bufflen);
			printf("setArg c'est une string : %s:\n", buff);

			tmp = mxCreateString(buff);
			}
		else
			{
			// copying double or array of doubles
			int m,n,i;
			double *x, *y;

			n = mxGetN(tmp);
			m = mxGetM(tmp);
			x = mxGetPr(tmp);
			tmp = mxCreateDoubleMatrix(m,n,mxREAL);
			y = mxGetPr(tmp);
			printf( "SetArgs c'est un tableau de %i, %i\n", n, m );
			for (i = 0; i < n * m; i++)
				y[i] = x[i];
			}

		// setting the value
		mxSetField(m_ArgClone,0,fnames[i],tmp);
		}

}		
		
// Get a mxarray output agrument from the Interface
mxArray *MLInterface::GetArgs()
{
	return m_ArgClone;
}

void MLInterface::PrintArg()
{
	int numDims, numFields, i, m, n;
	const int *Dims;
	char *Name;
	const mxArray *m_Arg = (m_Clone?m_ArgClone:m_ArgNorm);

	if( !mxIsStruct(m_Arg))
		{
		mexErrMsgTxt("Not a structure");
		return;
		}

	numDims = mxGetNumberOfDimensions(m_Arg);
	mexPrintf("# of dimensions = %i\n",numDims);
	Dims = mxGetDimensions(m_Arg);
	numFields = mxGetNumberOfFields(m_Arg);
	mexPrintf("# of fields = %i\n",numFields);
	for( i = 0; i < numFields; i++)
		{
		double	*Array;
		int		j;

		Name = (char *)mxGetFieldNameByNumber(m_Arg,i);
		mexPrintf("%s",Name);
		Array = ReadArray(Name, &m, &n);
		if (n*m > 1) mexPrintf("(%i,%i)\n",m, n);
		mexPrintf(" =");
		for( j=0; j < m*n; j++)
			mexPrintf(" %lf", Array[j]);
		mexPrintf("\n");
		}

}

// Read a double argument from Name
double MLInterface::ReadDouble( char *Name )
{
	const mxArray	*tmp;
	double			*Value;
	const mxArray	*m_Arg = (m_Clone?m_ArgClone:m_ArgNorm);

	Value = NULL;

	tmp = mxGetField(m_Arg, 0, Name);

	if(!tmp)
		{
		printf ( "Un clone ? %s\n", m_Clone?"True":"False");
		mexPrintf("Argument %s does not exist in structure \n", Name);
		return NULL;
		}

	if(!mxIsDouble(tmp))
		{
		mexPrintf("Argument %s is not a double\n", Name);
		return NULL;
		}

	if(mxGetN(tmp)*mxGetM(tmp)!=1)
		{
		mexPrintf("Argument %s is an array(%i,%i) not a scalar\n", Name,mxGetN(tmp),mxGetM(tmp));
		return NULL;
		}

	Value = mxGetPr(tmp);

	return *Value;
}

// Read a double Array from name, return the sizes of the array
double *MLInterface::ReadArray( char *Name, int *SizeM, int *SizeN )
{
	const mxArray	*tmp;
	double			*Value;
	const mxArray	*m_Arg = (m_Clone?m_ArgClone:m_ArgNorm);

	Value = NULL;
	*SizeM = 0;
	*SizeN = 0;

	tmp = mxGetField(m_Arg, 0, Name);

	if(!tmp)
		{
		printf("C'est un clone ? %i\n",(int)m_Clone);
		mexPrintf("Argument %s does not exist in structure \n", Name);
		return NULL;
		}

	if(!mxIsDouble(tmp))
		{
		mexPrintf("Argument %s is not a double\n", Name);
		return NULL;
		}

	Value = mxGetPr(tmp);
	*SizeN = mxGetN(tmp);
	*SizeM = mxGetM(tmp);

	return Value;
}

// Read a string from Name
char *MLInterface::ReadString( char *Name )
{
	const mxArray	*tmp;
	char			*Buffer;
	int				BuffLen,status;
	const mxArray *m_Arg = (m_Clone?m_ArgClone:m_ArgNorm);


	Buffer = NULL;

	tmp = mxGetField(m_Arg, 0, Name);
	if(!mxIsDouble(tmp))
		{
		BuffLen = mxGetN(tmp)*mxGetM(tmp)+1;
		Buffer = (char *)mxCalloc(BuffLen,sizeof(char));
		status = mxGetString(tmp,Buffer,BuffLen);
		if(status != 0)
			mexPrintf("String variable %s unreadable\n",Name);
		}
	else
		mexPrintf("Argument %s is not a string\n", Name);

	return Buffer;
}
